@@ -18,30 +18,34 @@ class SlappResponseObject:
         placements_for_players: Dict[str, Dict[str, List[Bracket]]] = {}
         """Dictionary keyed by Player id, of value Dictionary keyed by Source id of value Placements list"""
 
-        for team_id in response.get("AdditionalTeams"):
-            known_teams[team_id.__str__()] = Team.from_dict(response.get("AdditionalTeams")[team_id])
+        for team_id, team_dict in response.get("AdditionalTeams").items():
+            known_teams[team_id.__str__()] = Team.from_dict(team_dict)
         for team in matched_teams:
             known_teams[team.guid.__str__()] = team
 
-        matched_players_for_teams: Dict[str, List[Dict[str, Union[Player, bool]]]] = {}
-        for team_id in response.get("PlayersForTeams"):
-            matched_players_for_teams[team_id] = []
-            for tup in response.get("PlayersForTeams")[team_id]:
-                player_tuple_for_team: Dict[str, Union[Player, bool]] = \
-                    {"Item1": Player.from_dict(tup["Item1"]) if "Item1" in tup else None,
-                     "Item2": "Item2" in tup}
-                matched_players_for_teams[team_id].append(player_tuple_for_team)
+        # We're gonna tidy the mess that the serialization made.
+        # The tuple is sent as a dict, which is keyed by Item1, Item2, and contains the Player (Item1) or bool (Item2).
+        matched_players_for_teams: Dict[str, List[Tuple[Player, bool]]] = dict()
+        """Dictionary keyed by Team id, of value (Player, bool)[], where the bool is if the Player is currently in the team."""
+        for team_id, player_tuples in response.get("PlayersForTeams").items():
+            for tup in player_tuples:
+                player_dict = tup.get("Item1", None)
+                if isinstance(player_dict, dict):
+                    matched_players_for_teams\
+                        .setdefault(team_id, [])\
+                        .append((Player.from_dict(player_dict), bool(tup.get("Item2", False))))
 
         sources: Dict[str, SimpleSource] = {}
         for source_id, source_name in response.get("Sources").items():
             sources[source_id] = SimpleSource(id=source_id, name=source_name)
 
-        for player_id in response.get("PlacementsForPlayers"):
-            placements_for_players[player_id.__str__()] = {}
-            for source_id in response.get("PlacementsForPlayers")[player_id]:
-                placements_for_players[player_id][source_id] = []
-                for bracket in response.get("PlacementsForPlayers")[player_id][source_id]:
-                    placements_for_players[player_id][source_id].append(Bracket.from_dict(bracket))
+        for player_id, source_dicts in response.get("PlacementsForPlayers").items():
+            for source_id, brackets in source_dicts.items():
+                for bracket in brackets:
+                    placements_for_players\
+                        .setdefault(player_id, {})\
+                        .setdefault(source_id, [])\
+                        .append(Bracket.from_dict(bracket))
 
         self.matched_players = matched_players
         self.matched_teams = matched_teams
@@ -101,8 +105,8 @@ class SlappResponseObject:
     def get_players_in_team(self, team_guid: Union[UUID, str], include_ex_players: bool = True) -> List[Player]:
         """Return Player objects for the specified team id, optionally excluding players no longer in the team."""
 
-        return [player_dict["Item1"] for player_dict in self.matched_players_for_teams.get(team_guid.__str__(), [])
-                if player_dict and player_dict.get("Item1") and (player_dict["Item2"] or include_ex_players)]
+        return [tup[0] for tup in self.matched_players_for_teams.get(team_guid.__str__(), [])
+                if tup and (tup[1] or include_ex_players)]
 
     def get_team(self, team_id: Union[str, UUID]) -> Team:
         from slapp_py.core_classes.builtins import NoTeam, UnknownTeam
@@ -163,15 +167,21 @@ class SlappResponseObject:
         return result
 
     def get_grouped_sources_text(self, sources: Union[Player, Team, List[UUID]]) -> List[str]:
-        """Group the SimpleSource list for the specified sources/team/player by tourney name."""
+        """
+        Group the SimpleSource list for the specified sources/team/player by tourney name.
+        Duplicates are removed.
+        """
         groups = self.get_grouped_sources(sources)
         message = []
         for tourney, simple_sources in groups.items():
             group_key = tourney + ': ' if tourney else ''
-            group_value = (', '.join([simple_source.get_linked_date_display() for simple_source in simple_sources])
-                           if group_key
-                           else '\n'.join([simple_source.get_linked_name_display() for simple_source in simple_sources])
-                           )
+            group_value_array = list(set(
+                [simple_source.get_linked_date_display() for simple_source in simple_sources]
+                if group_key
+                else [simple_source.get_linked_name_display() for simple_source in simple_sources]
+            ))
+            separator = ', ' if group_key else '\n'
+            group_value = separator.join(group_value_array)
             message.append(f"{group_key}{group_value}")
         return message
 
@@ -183,7 +193,7 @@ class SlappResponseObject:
 
     def get_first_placements(self, p: Player) -> List[str]:
         """
-        Gets a list of displayed text in form {bracket.name} + ' in ' + {attempt_link_source(self.sources[source_id])}
+        Gets a list of displayed text in form {bracket.name} + ' in ' + {linked_source}
         where the specified player has come first.
         """
         result = []
